#include "dialog.h"
#include "ui_dialog.h"
#include <QDebug>
#include <QTimer>
#include <QDateTime>
#include <QDate>
#include <QTime>

#define ALERT  15

// The more you use an API the more bugs you find.

Dialog::Dialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::Dialog)
{
    ui->setupUi(this);
   // 100msec Timer
    QTimer *timer = new QTimer(this);
    connect(timer, SIGNAL(timeout()), this, SLOT(up_to_date()));
    timer->start(100);   //Reference: 1000msec = 1 sec, 3600000 = 1 hour, 21600000 = 6 hours

    blinker = new QTimer(this);
    connect(blinker, SIGNAL(timeout()), this, SLOT(blink_toggle()));

    ui->contest_CB->setEnabled(FALSE);

    ui->alert_PB->setDisabled(TRUE); // Contest is not live
    ui->alert_PB->setVisible(FALSE);
    active_icon = 2;
    warning_period = 900000; // 15 minutes = 900,000 msec
}

Dialog::~Dialog()
{
    delete ui;
}

void Dialog::up_to_date(){
    // Do UTC first
    utc_dt = utc_dt.currentDateTimeUtc();
    utc = utc_dt.time();
    utc_date = utc_dt.date();
    utc_time = utc.toString("HH:mm:ss");
    ui->UTC_time->setText(utc_time);
    u_date = utc_date.toString("d MMM yyyy");
    ui->UTC_date->setText(u_date);

    // Now do Local
    loco_dt = loco_dt.currentDateTime();
    loco = loco_dt.time();
    loco_date = loco_dt.date();
    loco_time = loco.toString("HH:mm:ss");
    ui->local_time->setText(loco_time);
    l_date = loco_date.toString("d MMM yyyy");
    ui->local_date->setText(l_date);
    checkvalid();
    if(utc.second() == 0)
        {
            //Only check to see if alarm time matches every minute.  ie., when seconds = 0
            run_checks();
        }
}

void Dialog::on_reset_PB_clicked()
{
    ui->alert_PB->setDisabled(TRUE); // Contest is not live
    ui->alert_PB->setVisible(FALSE);
    ui->start_dTE->setDateTime(QDateTime(QDate(2011, 1,1)));
    ui->end_dTE->setDateTime(QDateTime(QDate(2011, 1,1)));
    ui->contest_CB->setEnabled(FALSE);
    ui->contest_CB->setChecked(FALSE);
}

void Dialog::on_pushButton_clicked()
{
    utc_dt = utc_dt.currentDateTimeUtc();
    ui->start_dTE->setDateTime(utc_dt);
    ui->end_dTE->setDateTime(utc_dt);
}

void Dialog::run_checks()
{
    // If Qt did not have a bug in QDateTime comparisons I would not have to do this big work around
    //qDebug() << "DEBUG: Current Time = " <<utc_dt << " Alarm time = " << ui->start_dTE->dateTime() ;
checkvalid();

// Now check to see if date/time have reached the start date/time of the contest
 if(d_cur.operator >=(ds_alm))
    {

     // Functions that allow the alert icon to blink
         //Calculate duration of contest
        //Watch the data types!  These are 64bit variables.
        current_time = utc_dt.currentDateTimeUtc().toMSecsSinceEpoch(); // Just in case it forgot
         alarm_time = ui->end_dTE->dateTime().toMSecsSinceEpoch();
        qint64 st = ui->start_dTE->dateTime().toMSecsSinceEpoch();
        qint64 tt = alarm_time - current_time;
         //qDebug() << "DEBUG: Current time = " << current_time << " Start time = " << st << " Alarm/End time = " << alarm_time << " tt = " << tt;
         if(((alarm_time - current_time) < warning_period)  && ((alarm_time - current_time) >= 0) )
         {
             //Blink here
             ////qDebug() << "DEBUG: enable blinking icon";
             blink();
         }

    //qDebug() << "DEBUG: Current date is equal or greater than alarm date";
    if(t_cur.operator >= (ts_alm))
        {
            //qDebug() << "DEBUG: Current time is past alarm START time";
            // Now check to see if it is past END time
        if(d_cur.operator <=(de_alm))
        {
            if(t_cur.operator <=(te_alm))
            {
                ui->alert_PB->setDisabled(FALSE);
                ui->alert_PB->setIcon(QIcon(":/icons/green")); // Contest is live
                active_icon = 1;
                //qDebug() << "DEBUG: secs to = %d " << ui->start_dTE->dateTime().secsTo(ui->end_dTE->dateTime());
            }else
            {
                //Date is still OK but time is up
                ui->alert_PB->setIcon(QIcon(":/icons/red")); // Contest is over
                active_icon = 3;
            }

        }else
        {
            //Date is past end of contest
             ui->alert_PB->setIcon(QIcon(":/icons/red")); // Contest is over
             active_icon = 3;
        }


        }
    }
}


void Dialog::on_start_dTE_timeChanged(QTime date)
{
// All this to zero out the seconds!  What a pain in the butt.
    hr =date.hour();
    mn = date.minute();
    date.setHMS(hr, mn, 0, 0);
    ui->start_dTE->setTime(date);
}

void Dialog::on_end_dTE_timeChanged(QTime date)
{
    // All this to zero out the seconds!  What a pain in the butt.
        hr =date.hour();
        mn = date.minute();
        date.setHMS(hr, mn, 0, 0);
        ui->end_dTE->setTime(date);
}

void Dialog::on_contest_CB_toggled(bool checked)
{
    if(checked){
        //Make the icon visible and do checks
        ui->alert_PB->setVisible(TRUE);
    }else
    {
        ui->alert_PB->setVisible(FALSE);
    }

}

void Dialog::checkvalid()
{
    t_cur = utc_dt.time();
    d_cur = utc_dt.date();
    ts_alm = ui->start_dTE->time();
    ds_alm = ui->start_dTE->date();
    te_alm = ui->end_dTE->time();
    de_alm = ui->end_dTE->date();
    // Check to see if valid date/time information is set in the alarms
    if(ds_alm.operator <=(de_alm) && ts_alm.operator <(te_alm))
    {
        //Looks ok.  Enable the check box
        ui->contest_CB->setEnabled(TRUE);
    }else
    {
         ui->contest_CB->setEnabled(FALSE);
         return;
    }
}

void Dialog::blink()
{
    // Timer name is blinker
    if(blinker->isActive())
    {
        return;
    }else
    {
        //qDebug() << "DEBUG: blinker timer start";
        blinker->start(1000); //Reference: 1000msec = 1 sec
    }

}
void Dialog::blink_toggle()
{
   switch (active_icon)
   {
   case 1:
       // Currently green, set to yellow
       ui->alert_PB->setIcon(QIcon(":/icons/yellow"));
       active_icon = 2;
       break;
   case 2:
       //Currently yellow, set to green
       ui->alert_PB->setIcon(QIcon(":/icons/green"));
      active_icon = 1;
       break;
   case 3:
       // Currently red, leave it alone
       break;
   }
   }

